@@ -3,13 +3,12 @@ from collections import namedtuple
 from threading import RLock
 
 from mo_future import text
+from mo_imports import expect, export
+from mo_parsing.results import ParseResults, _flatten
 
-from mo_parsing.engine import Engine
 from mo_parsing.exceptions import ParseException
-from mo_parsing.results import ParseResults
 from mo_parsing.utils import Log, MAX_INT, wrap_parse_action, empty_tuple, is_forward
 
-# import later
 (
     SkipTo,
     Many,
@@ -18,7 +17,6 @@ from mo_parsing.utils import Log, MAX_INT, wrap_parse_action, empty_tuple, is_fo
     Optional,
     NotAny,
     Suppress,
-    _flatten,
     And,
     MatchFirst,
     Or,
@@ -28,8 +26,26 @@ from mo_parsing.utils import Log, MAX_INT, wrap_parse_action, empty_tuple, is_fo
     Literal,
     Token,
     Group,
-    regex_parameters,
-) = [None] * 18
+    regex_parameters
+) = expect(
+    "SkipTo",
+    "Many",
+    "ZeroOrMore",
+    "OneOrMore",
+    "Optional",
+    "NotAny",
+    "Suppress",
+    "And",
+    "MatchFirst",
+    "Or",
+    "MatchAll",
+    "Empty",
+    "StringEnd",
+    "Literal",
+    "Token",
+    "Group",
+    "regex_parameters"
+)
 
 DEBUG = False
 
@@ -81,7 +97,6 @@ class ParserElement(object):
         "parseAction",
         "parser_name",
         "token_name",
-        "engine",
         "streamlined",
         "min_length_cache",
         "parser_config",
@@ -92,7 +107,6 @@ class ParserElement(object):
         self.parseAction = list()
         self.parser_name = ""
         self.token_name = ""
-        self.engine = engine.CURRENT
         self.streamlined = False
         self.min_length_cache = -1
 
@@ -109,7 +123,6 @@ class ParserElement(object):
     def copy(self):
         output = object.__new__(self.__class__)
         le = self.parser_config.lock_engine
-        output.engine = le or engine.CURRENT
         output.parseAction = self.parseAction[:]
         output.parser_name = self.parser_name
         output.token_name = self.token_name
@@ -229,8 +242,7 @@ class ParserElement(object):
 
     def _parse(self, string, start, doActions=True):
         try:
-            index = self.engine.skip(string, start)
-            result = self.parseImpl(string, index, doActions)
+            result = self.parseImpl(string, start, doActions)
         except Exception as cause:
             self.parser_config.failAction and self.parser_config.failAction(
                 self, start, string, cause
@@ -239,7 +251,7 @@ class ParserElement(object):
 
         if doActions or self.parser_config.callDuringTry:
             for fn in self.parseAction:
-                next_result = fn(result, index, string)
+                next_result = fn(result, result.start, string)
                 if next_result.end < result.end:
                     Log.error(
                         "parse action not allowed to roll back the end of parsing"
@@ -275,17 +287,24 @@ class ParserElement(object):
         return self._parseString(string, parseAll=parseAll)
 
     def _parseString(self, string, parseAll=False):
-        # TODO: PUT THIS streamling IN THE ENTRY POINT
-        expr = self.streamline()
-        for e in expr.engine.ignore_list:
-            e.streamline()
+        # TODO: PUT THIS streamline IN THE ENTRY POINT
+        start = 0
+        e = expr = self.streamline()
+        while is_forward(e):
+            e = e.expr
+
+        eng = None
+        if isinstance(e, Many) or isinstance(e, And):
+            eng = e.parser_config.engine
+            start = eng.skip(string, start)
         if expr.token_name:
             # TOP LEVEL NAMES ARE NOT ALLOWED
             expr = Group(expr)
-        tokens = expr._parse(string, 0)
+        tokens = expr._parse(string, start)
         end = tokens.end
         if parseAll:
-            end = expr.engine.skip(string, end)
+            if eng:
+                end = eng.skip(string, end)
             StringEnd()._parse(string, end)
         return tokens
 
@@ -304,7 +323,8 @@ class ParserElement(object):
         end = 0
         matches = 0
         while end <= instrlen and matches < maxMatches:
-            start = self.engine.skip(string, end)
+            start = end
+            # start = self.engine.skip(string, end)
             try:
                 tokens = self._parse(string, start)
             except ParseException:
@@ -450,7 +470,7 @@ class ParserElement(object):
         if other is Ellipsis:
             return _PendingSkip(self)
 
-        return And([self, engine.CURRENT.normalize(other)]).streamline()
+        return And([self, engine.CURRENT.normalize(other)], engine.CURRENT).streamline()
 
     def __radd__(self, other):
         """
@@ -522,7 +542,9 @@ class ParserElement(object):
                 type(other[1]),
             )
 
-        ret = Many(self, min_match=minElements, max_match=maxElements).streamline()
+        ret = Many(
+            self, engine.CURRENT, min_match=minElements, max_match=maxElements
+        ).streamline()
         return ret
 
     def __rmul__(self, other):
@@ -604,16 +626,6 @@ class ParserElement(object):
         """
         return Suppress(self)
 
-    def leaveWhitespace(self):
-        """
-        Disables the skipping of whitespace before matching the characters in the
-        `ParserElement`'s defined pattern.  This is normally only used internally by
-        the mo_parsing module, but may be needed in some whitespace-sensitive grammars.
-        """
-        with Engine(""):
-            output = self.copy()
-        return output
-
     def __str__(self):
         return self.parser_name
 
@@ -695,17 +707,14 @@ class _PendingSkip(ParserElement):
         Log.error("use of `...` expression without following SkipTo target expression")
 
 
-# export
-from mo_parsing import engine, results
-
-engine.ParserElement = ParserElement
-results.ParserElement = ParserElement
-
 NO_PARSER = (
     ParserElement().set_parser_name("<nothing>")
 )  # USE THIS WHEN YOU DO NOT CARE ABOUT THE PARSER TYPE
 NO_RESULTS = ParseResults(NO_PARSER, -1, 0, [])
 
-results.NO_PARSER = NO_PARSER
-results.NO_RESULTS = NO_RESULTS
-del results
+
+export("mo_parsing.results", ParserElement)
+export("mo_parsing.results", NO_PARSER)
+export("mo_parsing.results", NO_RESULTS)
+
+from mo_parsing import engine
\ No newline at end of file
