@@ -112,11 +112,7 @@ _escapes.update({c: "\\" + c for c in r".^$*?+-{}[]\|()"})
 
 def regex_range(s, exclude=False):
     def esc(s):
-        r = re.escape(s)
-        o = _escapes.get(s, s)
-        if s not in "\t\r\n #~&" and r!=o:
-            Log.error("expecting same")
-        return o
+        return _escapes.get(s, s)
 
     if not s:
         return ""
@@ -241,11 +237,16 @@ builtin_lookup = {"".join.__name__: ("iterable",)}
 def is_forward(expr):
     return expr.__class__.__name__ == "Forward"
 
+
 def is_backtracking(expr):
     """
     RETURN true IF THIS CAN BE EXPENSIVE BACKTRACKER
     """
-    return isinstance(expr, Many) and expr.parser_config.max_match - expr.parser_config.min_match > 3
+    return (
+        isinstance(expr, Many)
+        and expr.parser_config.max_match - expr.parser_config.min_match > 3
+    )
+
 
 def forward_type(expr):
     """
